var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
import { mergeProps as _mergeProps, Fragment as _Fragment, createVNode as _createVNode } from "vue";
import { computed, defineComponent, ref, watch } from "vue";
import { ARROW_DOWN_IMG, ARROW_UP_IMG } from "./constant";
import { fillNumber, formatDate, makeArrayProp, makeDateProp, makeNumberProp, makeStringProp, pick, truthProp } from "./utils";
import languageUtil from "./language";
import { useExpose } from "./hooks";
import CalendarDate from "./CalendarDate";
import CalendarTime from "./CalendarTime";
import CalendarYearMonth from "./CalendarYearMonth";
const calendarProps = {
  themeColor: {
    type: Object,
    default: () => ({})
  },
  changeYearFast: Boolean,
  showArrow: Boolean,
  showWeekView: Boolean,
  visible: Boolean,
  disabledWeekView: Boolean,
  showAction: truthProp,
  pickerType: makeStringProp("datetime"),
  showTodayButton: truthProp,
  defaultDatetime: makeDateProp(new Date()),
  minDate: makeDateProp(null),
  maxDate: makeDateProp(null),
  format: makeStringProp(""),
  model: makeStringProp("inline"),
  markType: makeStringProp("dot"),
  markDate: makeArrayProp(),
  disabledDate: {
    type: Function,
    default: () => false
  },
  disabledTime: {
    type: Function,
    default: () => false
  },
  lang: makeStringProp("CN"),
  scrollChangeDate: truthProp,
  minuteStep: makeNumberProp(1),
  weekStart: makeStringProp("Sunday"),
  disabledScroll: {
    type: [Boolean, String],
    default: () => false
  },
  showNotCurrentMonthDay: truthProp,
  firstDayOfMonthClassName: makeStringProp(""),
  todayClassName: makeStringProp(""),
  checkedDayClassName: makeStringProp(""),
  notCurrentMonthDayClassName: makeStringProp(""),
  disabledClassName: makeStringProp("")
};
var stdin_default = defineComponent({
  name: "VueHashCalendar",
  props: calendarProps,
  emits: ["update:visible", "update:showWeekView", "calendarTypeChange", "click", "confirm", "change", "slidechange", "touchstart", "touchmove", "touchend"],
  setup(props, {
    emit,
    slots
  }) {
    const defaultDate = {
      year: new Date().getFullYear(),
      month: new Date().getMonth(),
      day: new Date().getDate(),
      hours: new Date().getHours(),
      minutes: new Date().getMinutes()
    };
    const calendarRef = ref();
    const checkedDate = ref(defaultDate);
    const isShowCalendar = ref(false);
    const isShowWeek = ref(false);
    const calendarBodyHeight = ref(0);
    const firstTimes = ref(true);
    const currDateTime = ref(new Date());
    const yearMonthType = ref("date");
    const isShowDatetimePicker = computed({
      get() {
        return props.model === "inline" ? true : props.visible;
      },
      set(val) {
        emit("update:visible", val);
      }
    });
    const isShowArrowImg = computed(() => props.showArrow && props.model === "inline");
    const calendarContentHeight = computed(() => {
      if (props.pickerType === "time") {
        return 245;
      }
      return calendarBodyHeight.value;
    });
    const slideChange = (direction) => {
      emit("slidechange", direction);
    };
    const toggleWeek = () => {
      isShowWeek.value = !isShowWeek.value;
      if (isShowWeek.value)
        slideChange("up");
      else
        slideChange("down");
    };
    const today = () => {
      var _a;
      if (props.disabledDate(new Date()))
        return;
      (_a = calendarRef.value) == null ? void 0 : _a.today();
    };
    const lastMonth = () => {
      var _a;
      (_a = calendarRef.value) == null ? void 0 : _a.getLastMonth();
    };
    const nextMonth = () => {
      var _a;
      (_a = calendarRef.value) == null ? void 0 : _a.getNextMonth();
    };
    const lastWeek = () => {
      var _a, _b;
      (_a = calendarRef.value) == null ? void 0 : _a.getLastMonth();
      (_b = calendarRef.value) == null ? void 0 : _b.changeWeekView({
        isNext: false
      });
    };
    const nextWeek = () => {
      var _a, _b;
      (_a = calendarRef.value) == null ? void 0 : _a.getNextMonth();
      (_b = calendarRef.value) == null ? void 0 : _b.changeWeekView({
        isNext: true
      });
    };
    const dateChange = (date) => {
      date.hours = checkedDate.value.hours;
      date.minutes = checkedDate.value.minutes;
      checkedDate.value = date;
    };
    const dateClick = (date, type) => {
      date.hours = checkedDate.value.hours;
      date.minutes = checkedDate.value.minutes;
      checkedDate.value = date;
      let fDate = new Date(`${checkedDate.value.year}/${checkedDate.value.month + 1}/${checkedDate.value.day} ${checkedDate.value.hours}:${checkedDate.value.minutes}`);
      if (props.format) {
        fDate = formatDate(fDate, props.format, props.lang);
      }
      if (type) {
        switch (type) {
          case "yearRange":
            yearMonthType.value = "year";
            break;
          case "year":
            yearMonthType.value = "month";
            break;
          case "month":
            currDateTime.value = new Date(fDate);
            yearMonthType.value = "date";
            break;
        }
        emit("calendarTypeChange", yearMonthType.value);
      }
      emit("click", fDate);
    };
    const timeChange = (date) => {
      date.year = checkedDate.value.year;
      date.month = checkedDate.value.month;
      date.day = checkedDate.value.day;
      checkedDate.value = date;
    };
    const close = () => {
      isShowDatetimePicker.value = false;
    };
    const confirm = () => {
      let date = new Date(`${checkedDate.value.year}/${checkedDate.value.month + 1}/${checkedDate.value.day} ${checkedDate.value.hours}:${checkedDate.value.minutes}`);
      if (props.format) {
        date = formatDate(date, props.format, props.lang);
      }
      emit("confirm", date);
      if (props.model === "dialog") {
        close();
      }
    };
    const formatDatetime = (time, format) => formatDate(time, format, props.lang);
    const showYearMonthPicker = () => {
      if (!props.changeYearFast)
        return;
      if (yearMonthType.value === "date") {
        yearMonthType.value = "month";
      } else if (yearMonthType.value === "month") {
        yearMonthType.value = "year";
      } else if (yearMonthType.value === "year") {
        yearMonthType.value = "yearRange";
      } else {
        yearMonthType.value = "date";
      }
      emit("calendarTypeChange", yearMonthType.value);
    };
    const showCalendar = () => {
      if (isShowCalendar.value) {
        showYearMonthPicker();
      } else {
        emit("calendarTypeChange", "date");
      }
      isShowCalendar.value = true;
    };
    const showTime = () => {
      isShowCalendar.value = false;
      emit("calendarTypeChange", "time");
      yearMonthType.value = "date";
    };
    const heightChange = (height) => {
      if (!firstTimes.value && props.model === "dialog")
        return;
      calendarBodyHeight.value = height;
      firstTimes.value = false;
    };
    const getThemeColor = () => {
      const cssVar = {};
      if (props.themeColor) {
        const themeColorKeys = Object.keys(props.themeColor || {});
        if (themeColorKeys.length) {
          themeColorKeys.forEach((k) => {
            cssVar[`--hash-calendar-${k}`] = props.themeColor[k];
          });
        }
      }
      return cssVar;
    };
    const touchStart = (event) => {
      emit("touchstart", event);
    };
    const touchMove = (event) => {
      emit("touchmove", event);
    };
    const touchEnd = (event) => {
      emit("touchend", event);
    };
    watch(() => props.defaultDatetime, (val) => {
      currDateTime.value = val;
    }, {
      immediate: true
    });
    watch(() => props.pickerType, (val) => {
      if (val === "time") {
        showTime();
      } else {
        showCalendar();
      }
    }, {
      immediate: true
    });
    watch(checkedDate, () => {
      let date = new Date(`${checkedDate.value.year}/${checkedDate.value.month + 1}/${checkedDate.value.day} ${checkedDate.value.hours}:${checkedDate.value.minutes}`);
      if (props.format) {
        date = formatDate(date, props.format, props.lang);
      }
      emit("change", date);
    }, {
      deep: true
    });
    watch(() => props.showWeekView, (val) => {
      isShowWeek.value = val;
    }, {
      immediate: true
    });
    useExpose({
      today,
      lastMonth,
      nextMonth,
      lastWeek,
      nextWeek
    });
    const renderTodayButton = () => {
      let todayEle = languageUtil[props.lang].TODAY;
      if (slots.today) {
        todayEle = slots.today();
      }
      if (props.showTodayButton) {
        return _createVNode("div", {
          "class": `calendar_confirm ${props.disabledDate(new Date()) ? "today_disable" : ""}`,
          "onClick": today
        }, [todayEle]);
      }
    };
    const renderConfirmButton = () => {
      let confirmEle = languageUtil[props.lang].CONFIRM;
      if (slots.confirm) {
        confirmEle = slots.confirm();
      }
      if (props.model === "dialog") {
        return _createVNode("div", {
          "class": "calendar_confirm",
          "onClick": confirm
        }, [confirmEle]);
      }
    };
    const renderAction = () => {
      if (props.showAction) {
        return _createVNode("div", {
          "class": "calendar_title",
          "onClick": (e) => {
            e.preventDefault();
            e.stopPropagation();
          },
          "style": {
            bottom: props.model === "dialog" ? `${calendarContentHeight.value}px` : "unset"
          }
        }, [slots.action ? slots.action() : _createVNode(_Fragment, null, [_createVNode("div", {
          "class": "calendar_title_date"
        }, [props.pickerType !== "time" ? _createVNode("span", {
          "class": `calendar_title_date_year ${isShowCalendar.value ? "calendar_title_date_active" : ""}`,
          "onClick": showCalendar
        }, [formatDatetime(`${checkedDate.value.year}/${checkedDate.value.month + 1}/${checkedDate.value.day}`, languageUtil[props.lang].DEFAULT_DATE_FORMAT)]) : null, props.pickerType !== "date" ? _createVNode("span", {
          "class": `calendar_title_date_time ${!isShowCalendar.value ? "calendar_title_date_active" : ""}`,
          "onClick": showTime
        }, [formatDatetime(`${checkedDate.value.year}/${checkedDate.value.month + 1}/${checkedDate.value.day} ${fillNumber(checkedDate.value.hours)}:${fillNumber(checkedDate.value.minutes)}`, languageUtil[props.lang].DEFAULT_TIME_FORMAT)]) : null]), renderTodayButton(), renderConfirmButton()])]);
      }
      return null;
    };
    const updateShowWeekView = (val) => {
      if (val) {
        emit("calendarTypeChange", "week");
      } else {
        emit("calendarTypeChange", "month");
      }
      isShowWeek.value = val;
    };
    const renderCalendar = () => _createVNode(CalendarDate, _mergeProps({
      "ref": calendarRef,
      "show": isShowCalendar.value,
      "onHeight": heightChange,
      "defaultDate": currDateTime.value,
      "onTouchstart": touchStart,
      "onTouchmove": touchMove,
      "onTouchend": touchEnd,
      "onSlidechange": slideChange,
      "onChange": dateChange,
      "onClick": dateClick,
      "showWeekView": isShowWeek.value,
      "onUpdate:showWeekView": updateShowWeekView
    }, pick(props, ["minDate", "maxDate", "disabledWeekView", "markType", "markDate", "disabledDate", "lang", "weekStart", "disabledScroll", "scrollChangeDate", "showNotCurrentMonthDay", "firstDayOfMonthClassName", "todayClassName", "checkedDayClassName", "notCurrentMonthDayClassName", "disabledClassName"])), pick(slots, ["week", "day"]));
    const renderTimePicker = () => {
      if (props.pickerType !== "date") {
        return _createVNode(CalendarTime, _mergeProps({
          "show": !isShowCalendar.value,
          "defaultTime": currDateTime.value,
          "calendarDate": checkedDate.value,
          "onChange": timeChange
        }, pick(props, ["minuteStep", "disabledTime"])), null);
      }
    };
    const renderYearMonthPicker = () => _createVNode(CalendarYearMonth, _mergeProps({
      "calendarContentHeight": calendarContentHeight.value,
      "calendarDate": checkedDate.value,
      "type": yearMonthType.value,
      "onTouchstart": touchStart,
      "onTouchmove": touchMove,
      "onTouchend": touchEnd,
      "onSlidechange": slideChange,
      "onClick": dateClick
    }, pick(props, ["minDate", "maxDate", "disabledDate", "lang", "disabledScroll", "checkedDayClassName", "notCurrentMonthDayClassName", "disabledClassName"])), null);
    const renderCtrlImg = () => {
      let confirmEle = _createVNode("img", {
        "src": isShowWeek.value ? ARROW_DOWN_IMG : ARROW_UP_IMG
      }, null);
      if (slots.arrow) {
        confirmEle = slots.arrow({
          isShowWeek: isShowWeek.value
        });
      }
      if (isShowArrowImg.value) {
        return _createVNode("div", {
          "class": "ctrl-img",
          "onClick": toggleWeek,
          "style": {
            "margin-top": props.model === "dialog" ? `${calendarContentHeight.value}px` : "unset"
          }
        }, [confirmEle]);
      }
    };
    const renderHashCalendar = () => {
      if (isShowDatetimePicker.value || props.model === "inline") {
        return _createVNode("div", {
          "class": `hash-calendar ${props.model === "inline" ? "calendar_inline" : ""}`,
          "style": __spreadValues({}, getThemeColor()),
          "onClick": close
        }, [renderAction(), _createVNode("div", {
          "class": "calendar_content",
          "style": {
            height: `${calendarContentHeight.value}px`
          },
          "onClick": (e) => {
            e.preventDefault();
            e.stopImmediatePropagation();
          }
        }, [props.pickerType !== "time" ? renderCalendar() : "", renderTimePicker(), props.changeYearFast ? renderYearMonthPicker() : ""]), renderCtrlImg()]);
      }
    };
    return () => renderHashCalendar();
  }
});
export {
  calendarProps,
  stdin_default as default
};
